package com.schbrain.archetype.initializer.maven;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.schbrain.archetype.initializer.param.ArchetypeGenerateParam;
import com.schbrain.common.util.JacksonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.maven.cli.MavenCli;

import java.io.File;

/**
 * @author liaozan
 * @since 2022/3/20
 */
@Slf4j
public class MavenUtils {

    public static File generate(ArchetypeGenerateParam param) {
        log.info("Prepare to generate archetype project: {}", JacksonUtils.toJsonString(param, true));

        MavenCli mavenCli = new MavenCli();

        File archetype = getArchetypeDirectory(param.getArtifact());
        String outputDirectory = archetype.getAbsolutePath();

        String[] args = getArgs(param, outputDirectory);

        System.getProperties().setProperty("maven.multiModuleProjectDirectory", "$M2_HOME");
        mavenCli.doMain(args, null, System.out, System.err);

        log.info("Generate archetype project at {}", outputDirectory);
        return ZipUtil.zip(archetype);
    }

    private static File getArchetypeDirectory(String artifactId) {
        String tmpDirPath = FileUtil.getTmpDirPath();
        File archetype = new File(tmpDirPath, artifactId);
        // clean cache
        FileUtil.del(archetype);
        return archetype;
    }

    @SuppressWarnings("SpellCheckingInspection")
    private static String[] getArgs(ArchetypeGenerateParam param, String outputDirectory) {
        return new String[]{
                "-B",
                "-X",
                "archetype:generate",
                "-DarchetypeGroupId=com.schbrain.kp",
                "-DarchetypeArtifactId=schbrain-archetype",
                "-DarchetypeVersion=1.0-SNAPSHOT",
                String.format("-DoutputDirectory=%s", outputDirectory),
                String.format("-DgroupId=%s", param.getGroup()),
                String.format("-DartifactId=%s", param.getArtifact()),
                String.format("-DpackageName=%s", param.getPackageName()),
                String.format("-DprojectName=%s", param.getProjectName()),
        };
    }

}